;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                        ;;
;;             Center for Spoken Language Understanding                   ;;
;;        Oregon Graduate Institute of Science & Technology               ;;
;;                         Portland, OR USA                               ;;
;;                        Copyright (c) 1999                              ;;
;;                                                                        ;;
;;      This module is not part of the CSTR/University of Edinburgh       ;;
;;               release of the Festival TTS system.                      ;;
;;                                                                        ;;
;;  In addition to any conditions disclaimers below, please see the file  ;;
;;  "license_ogi_tts.txt" distributed with this software for information  ;;
;;  on usage and redistribution, and for a DISCLAIMER OF ALL WARRANTIES.  ;;
;;                                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require 'ogi_wordsynth)

;;; voice definition

;;; Use the grouped file by default
(defvar mwmword_grouped_or_ungrouped 'grouped)

;; Location at runtime
(defvar mwm_word_dir (cdr (assoc 'mwm_word voice-locations)))
(set! load-path (cons (path-append mwm_word_dir "festvox") load-path))
(require 'mm3_durs)

(define (voice_mwm_word)
  (voice_mwm_diphone)  ;; inherit all this

  ;; Duration prediction
  (set! duration_cart_tree mm3_duration_cart_tree)
  (set! duration_ph_info mm3_durs)
  (Parameter.set 'Duration_Method Duration_Tree_ZScores)
  (Parameter.set 'Duration_Stretch 1.0)

  (mwm_word_reactivate)
  (set! fallback_voice_cmd mwm_word_fallback)
  (set! reactivate_voice_cmd mwm_word_reactivate)
  'mwm_word)

(define (mwm_word_fallback)
  (Parameter.set 'Synth_Method 'OGIdiphone)
  (OGIdbase.activate "mwm_diphone")      
  (OGIresLPC.init mwmdi_OGI_syn_params)  
  (set! current_voice_reset mwmdi_voice_reset)
  (set! current-voice 'mwm_diphone)  
  t)

(define (mwm_word_reactivate)
  (Parameter.set 'Synth_Method 'OGIword)
  (OGIdbase.activate "mwm_word")      
  (OGIresLPC.init mwm_word_syn_params)  
  (set! current_voice_reset mwmdi_voice_reset)
  (set! current-voice 'mwm_word)  
  t)



(set! mwm_word_syn_params 
 (list
  '(F0_default 50.0)      ;; default Fo used if can't find any Fo targets
  '(T0_UV_thresh 0.020)   ;; used to make V/UV decision 
  '(T0_UV_pm 0.010)       ;; used to place UV pmarks
  '(post_gain 1.0)        ;; adjust final loudness
  '(deemphasis 0.955)      ;; opposite of preemphasis (lowpass filter)
  '(mod_method "direct")  ;; method for realizing prosodic targets
;  '(mod_method "none")  ;; method for realizing prosodic targets
  '(window_type "trapezoid") ;; ola window
  
  ))

(define (init_mwm_word)
"(init_mwm_word)
  Initialise the MWM word database.  This sets up the 16K version
  for residual excited LPC."
(if (equal? mwmword_grouped_or_ungrouped 'ungrouped)

    ;;  ungrouped
    (OGIdbase.init
     (list
      '(dbname mwm_word)
     (list 'unitdic_file "/home/macon/TMOS/dic/worddic")
      '(phoneset "worldbet")  
      '(base_dir "/home/macon/TMOS/")
      '(lpc_dir "lpc/")
      '(lpc_ext ".lsf")
      '(exc_dir "lpc/")
      '(exc_ext ".res")
      '(pm_dir "pm/")
      '(pm_ext ".pmv")
      '(data_type "resLPC")
      '(access_mode "ondemand")
      '(samp_freq 22050)
      '(sig_band 0.010)
      '(isCompressed "Y") ;; if "Y", compress when saving group file
      '(preemph 0.96)  
      ))

    ;;  grouped -- parameters set here override those in groupfile
    (OGIdbase.init   
     (list
      '(dbname mwm_word)
      (list 'groupfile (path-append mwm_word_dir "group" "mwm_word_resLPC.group"))
      (list 'groupfile "/home/macon/TMOS/m.group")
      '(access_mode "ondemand")
      )))
    t)

(init_mwm_word)






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  make unitdic for word synthesizer
;;;
(define (build_worddic uttpath outfile)
"(build_worddic uttpath outfile) - make unit dic for word synthesizer"
;  (set! outfile "./worddic")
;  (set! uttpath "../utt/")
  (set! flist (directory-entries uttpath))
  (set! fpout (fopen outfile 'w))
  
  (mapcar 
   (lambda (uttfile) 
     (set! utt0 (utt.load nil (path-append uttpath uttfile)))
     
     ;; unit name: first word
     (set! uname (item.name (car (utt.relation.items utt0 "Word"))))
     
     ;; base file name
     (set! bsnm (path-basename uttfile))
     
     ;; neighbors
     (set! Lnb -1)
     (set! Rnb -1)
     (set! LisB "N")
     (set! RisB "N")

     ;; print entry
     (format fpout "%s %s %d %d %s %s " uname bsnm Lnb Rnb LisB RisB)

     ;; loop through segments, print 
     (format fpout "%d " (* 1000 (read-from-string (item.feat (car (utt.relation.items utt0 "Segment")) "begin"))))
     (mapcar
      (lambda (segitem)
	(format fpout "%s %d " (item.name segitem) (* 1000 (read-from-string (item.feat segitem "end"))))
	)
      (utt.relation.items utt0 "Segment"))
     (format fpout "\n") ;; end line
     )
   flist)
  t)

(proclaim_voice
 'mwm_word
 '((language english)
   (gender male)
   (dialect american)
   (description
    "This voice provides an American English male voice using a
     residual excited LPC synthesis module created at OGI.
     The voice is optimized to perform well for isolated words - it tries
     to find the desired word in a table of 1400 recorded whole words.
     When the whole word is not present in the table it temporarily
     falls back to diphone synthesis.  The duration model is trained on the
     isolated words, so the durations sound like those of slowly-pronounced
     isolated words.")))

(provide 'mwm_word)
